@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: demux_asf.c,v 1.188 2006/11/29 21:26:52 dgp85 Exp $
+ * $Id: demux_asf.c,v 1.192 2007/01/19 01:05:24 dgp85 Exp $
  *
  * demultiplexer for asf streams
  *
@@ -408,7 +408,9 @@ static int asf_read_header (demux_asf_t *this) {
     this->length = 0;
     
   /* compute average byterate (needed for seeking) */
-  if (this->length)
+  if (this->asf_header->file->max_bitrate)
+    this->rate = this->asf_header->file->max_bitrate >> 3;
+  else if (this->length)
     this->rate = (int64_t) this->input->get_length(this->input) * 1000 / this->length;
 
   _x_stream_info_set(this->stream, XINE_STREAM_INFO_BITRATE, this->asf_header->file->max_bitrate);
@@ -1354,7 +1356,6 @@ static int demux_asf_parse_http_references( demux_asf_t *this) {
   int             buf_used = 0;
   int             len;
   char           *href = NULL;
-  char           *mrl;
   int             free_href = 0;
 
   /* read file to memory.
@@ -1379,7 +1380,7 @@ static int demux_asf_parse_http_references( demux_asf_t *this) {
   ptr = buf;
   if (!strncmp(ptr, "[Reference]", 11)) {
 
-    mrl = this->input->get_mrl(this->input);
+    const char *const mrl = this->input->get_mrl(this->input);
     if (!strncmp(mrl, "http", 4)) {
       /* never trust a ms server, reopen the same mrl with the mms input plugin
        * some servers are badly configured and return a incorrect reference.
@@ -1789,10 +1790,6 @@ static int demux_asf_seek (demux_plugin_t *this_gen,
   int            i, state;
   int64_t        ts;
 
-  start_time /= 1000;
-  start_pos = (off_t) ( (double) start_pos / 65535 *
-              this->input->get_length (this->input) );
-
   lprintf ("demux_asf_seek: start_pos=%lld, start_time=%d\n",
 	   start_pos, start_time);
 
@@ -1824,6 +1821,10 @@ static int demux_asf_seek (demux_plugin_t *this_gen,
 
     _x_demux_flush_engine(this->stream);
     
+    start_time /= 1000;
+    start_pos = (off_t) ( (double) start_pos / 65535 *
+                this->input->get_length (this->input) );
+    
     if ( (!start_pos) && (start_time))
       start_pos = start_time * this->rate;
 
@@ -1951,6 +1952,22 @@ static int demux_asf_seek (demux_plugin_t *this_gen,
       this->streams[this->audio_stream].resync = 1;
       this->streams[this->audio_stream].skip   = 1;
     }
+  } else if (!playing && this->input->seek_time != NULL) {    
+    if (start_pos && !start_time)
+      start_time = this->length * start_pos / 65535;
+      
+    this->input->seek_time (this->input, start_time, SEEK_SET);
+    
+    this->keyframe_ts = 0;
+    this->keyframe_found = 0; /* means next keyframe */
+    if (this->video_stream >= 0) {
+      this->streams[this->video_stream].resync = 1;
+      this->streams[this->video_stream].skip   = 1;
+    }
+    if (this->audio_stream >= 0) {
+      this->streams[this->audio_stream].resync = 0;
+      this->streams[this->audio_stream].skip   = 0;
+    }
   } else {
     /* "streaming" mode */
     this->keyframe_ts = 0;
@@ -2034,15 +2051,8 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen,
     break;
 
   case METHOD_BY_EXTENSION: {
-    char *ending, *mrl;
-
-    mrl = input->get_mrl (input);
-
-    /*
-     * check extension
-     */
-
-    ending = strrchr (mrl, '.');
+    const char *const mrl = input->get_mrl (input);
+    const char *const ending = strrchr (mrl, '.');
 
     if (!ending)
       return NULL;
@@ -2105,20 +2115,20 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen,
   return &this->demux_plugin;
 }
 
-static char *get_description (demux_class_t *this_gen) {
+static const char *get_description (demux_class_t *this_gen) {
   return "ASF demux plugin";
 }
 
-static char *get_identifier (demux_class_t *this_gen) {
+static const char *get_identifier (demux_class_t *this_gen) {
   return "ASF";
 }
 
-static char *get_extensions (demux_class_t *this_gen) {
+static const char *get_extensions (demux_class_t *this_gen) {
   /* asx, wvx, wax are metafile or playlist */
   return "asf wmv wma asx wvx wax";
 }
 
-static char *get_mimetypes (demux_class_t *this_gen) {
+static const char *get_mimetypes (demux_class_t *this_gen) {
 
   return "video/x-ms-asf: asf: ASF stream;"
          "video/x-ms-wmv: wmv: Windows Media Video;"
