@@ -36,7 +36,7 @@
  * * ANIM (Animations)
  *   - Animation works fine, without seeking.
  *
- * $Id: demux_iff.c,v 1.16 2006/06/02 22:18:56 dsalt Exp $
+ * $Id: demux_iff.c,v 1.19 2007/01/19 00:26:40 dgp85 Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -827,7 +827,8 @@ static int demux_iff_send_chunk(demux_plugin_t *this_gen) {
               }
             } else {
               for (j = 0, k = (interleave_index / 2); j < (buf->size / 2); j += this->audio_channels) {
-                zw_16                   = BE_16(&pointer16_from[k++]);
+                zw_16                   = BE_16(&pointer16_from[k]);
+                k++;
                 zw_rescale              = zw_16;
                 zw_rescale             *= this->audio_volume_left;
                 zw_rescale             /= max_volume;
@@ -856,7 +857,8 @@ static int demux_iff_send_chunk(demux_plugin_t *this_gen) {
               }
             } else {
               for (j = 1; j < (buf->size / 2); j += this->audio_channels) {
-                zw_16                   = BE_16(&pointer16_from[k++]);
+                zw_16                   = BE_16(&pointer16_from[k]);
+                k++;
                 zw_rescale              = zw_16;
                 zw_rescale             *= this->audio_volume_left;
                 zw_rescale             /= max_volume;
@@ -1253,7 +1255,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   switch (stream->content_detection_method) {
 
     case METHOD_BY_EXTENSION: {
-      char *extensions, *mrl;
+      const char *extensions, *mrl;
 
       mrl                               = input->get_mrl (input);
       extensions                        = class_gen->get_extensions (class_gen);
@@ -1283,19 +1285,19 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   return &this->demux_plugin;
 }
 
-static char *get_description (demux_class_t *this_gen) {
+static const char *get_description (demux_class_t *this_gen) {
   return "IFF demux plugin";
 }
 
-static char *get_identifier (demux_class_t *this_gen) {
+static const char *get_identifier (demux_class_t *this_gen) {
   return "IFF";
 }
 
-static char *get_extensions (demux_class_t *this_gen) {
+static const char *get_extensions (demux_class_t *this_gen) {
   return "iff svx 8svx 16sv ilbm ham ham6 ham8 anim anim3 anim5 anim7 anim8";
 }
 
-static char *get_mimetypes (demux_class_t *this_gen) {
+static const char *get_mimetypes (demux_class_t *this_gen) {
   return "audio/x-8svx: 8svx: IFF-8SVX Audio;"
          "audio/8svx: 8svx: IFF-8SVX Audio;"
          "audio/x-16sv: 16sv: IFF-16SV Audio;"
@@ -1334,7 +1336,7 @@ static const demuxer_info_t demux_info_iff = {
   10                       /* priority */
 };
 
-const plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_DEMUX, 26, "iff", XINE_VERSION_CODE, &demux_info_iff, init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
