@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2002 the xine project
+ * Copyright (C) 2001-2005 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -16,12 +16,13 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
+ */
+
+/*
  * MS WAV File Demuxer by Mike Melanson (melanson@pcisys.net)
  * based on WAV specs that are available far and wide
  *
- * $Id: demux_wav.c,v 1.36 2003/01/23 16:12:15 miguelfreitas Exp $
- *
+ * $Id: demux_wav.c,v 1.64 2007/01/19 00:26:40 dgp85 Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -40,25 +41,22 @@
 #include "demux.h"
 #include "buffer.h"
 #include "bswap.h"
+#include "group_audio.h"
 
 #define WAV_SIGNATURE_SIZE 16
 /* this is the hex value for 'data' */
 #define data_TAG 0x61746164
 #define PCM_BLOCK_ALIGN 1024
 
-typedef struct {
+#define PREFERED_BLOCK_SIZE 4096
 
+typedef struct {
   demux_plugin_t       demux_plugin;
 
   xine_stream_t       *stream;
-
-  config_values_t     *config;
-
   fifo_buffer_t       *video_fifo;
   fifo_buffer_t       *audio_fifo;
-
   input_plugin_t      *input;
-
   int                  status;
 
   xine_waveformatex    *wave;
@@ -69,32 +67,23 @@ typedef struct {
   off_t                data_size;
 
   int                  seek_flag;  /* this is set when a seek just occurred */
-
-  char                 last_mrl[1024];
 } demux_wav_t;
 
 typedef struct {
-
   demux_class_t     demux_class;
-
-  /* class-wide, global variables here */
-
-  xine_t           *xine;
-  config_values_t  *config;
 } demux_wav_class_t;
 
+static int demux_wav_get_stream_length (demux_plugin_t *this_gen);
+
 /* returns 1 if the WAV file was opened successfully, 0 otherwise */
 static int open_wav_file(demux_wav_t *this) {
-
-  unsigned char signature[WAV_SIGNATURE_SIZE];
-  unsigned int chunk_tag;
-  unsigned int chunk_size;
-  unsigned char chunk_preamble[8];
+  uint8_t signature[WAV_SIGNATURE_SIZE];
+  uint32_t chunk_tag;
+  uint32_t chunk_size;
+  uint8_t chunk_preamble[8];
 
   /* check the signature */
-  this->input->seek(this->input, 0, SEEK_SET);
-  if (this->input->read(this->input, signature, WAV_SIGNATURE_SIZE) !=
-    WAV_SIGNATURE_SIZE)
+  if (_x_demux_read_header(this->input, signature, WAV_SIGNATURE_SIZE) != WAV_SIGNATURE_SIZE)
     return 0;
 
   if ((signature[0] != 'R') ||
@@ -111,6 +100,9 @@ static int open_wav_file(demux_wav_t *this) {
       (signature[15] != ' '))
     return 0;
 
+  /* file is qualified; skip over the header bytes in the stream */
+  this->input->seek(this->input, WAV_SIGNATURE_SIZE, SEEK_SET);
+
   /* go after the format structure */
   if (this->input->read(this->input,
     (unsigned char *)&this->wave_size, 4) != 4)
@@ -119,26 +111,37 @@ static int open_wav_file(demux_wav_t *this) {
   this->wave = (xine_waveformatex *) malloc( this->wave_size );
     
   if (this->input->read(this->input, (void *)this->wave, this->wave_size) !=
-    this->wave_size)
+    this->wave_size) {
+    free (this->wave);
     return 0;
-  xine_waveformatex_le2me(this->wave);
-  this->audio_type = formattag_to_buf_audio(this->wave->wFormatTag);
+  }
+  _x_waveformatex_le2me(this->wave);
+  this->audio_type = _x_formattag_to_buf_audio(this->wave->wFormatTag);
   if(!this->audio_type) {
     this->audio_type = BUF_AUDIO_UNKNOWN;
   }
 
+  if (this->wave->nChannels <= 0) {
+    free (this->wave);
+    return 0;
+  }
+
   /* traverse through the chunks to find the 'data' chunk */
   this->data_start = this->data_size = 0;
   while (this->data_start == 0) {
 
-    if (this->input->read(this->input, chunk_preamble, 8) != 8)
+    if (this->input->read(this->input, chunk_preamble, 8) != 8) {
+      free (this->wave);
       return 0;
+    }
     chunk_tag = LE_32(&chunk_preamble[0]);      
     chunk_size = LE_32(&chunk_preamble[4]);
 
     if (chunk_tag == data_TAG) {
       this->data_start = this->input->get_current_pos(this->input);
-      this->data_size = chunk_size;
+      /* Get the data length from the file itself, instead of the data
+       * TAG, for broken files */
+      this->data_size = this->input->get_length(this->input);
     } else {
       this->input->seek(this->input, chunk_size, SEEK_CUR);
     }
@@ -148,8 +151,8 @@ static int open_wav_file(demux_wav_t *this) {
 }
 
 static int demux_wav_send_chunk(demux_plugin_t *this_gen) {
-
   demux_wav_t *this = (demux_wav_t *) this_gen;
+
   buf_element_t *buf = NULL;
   unsigned int remaining_sample_bytes;
   off_t current_file_pos;
@@ -157,8 +160,13 @@ static int demux_wav_send_chunk(demux_plugin_t *this_gen) {
 
   /* just load data chunks from wherever the stream happens to be
    * pointing; issue a DEMUX_FINISHED status if EOF is reached */
-  remaining_sample_bytes = this->wave->nBlockAlign;
-  current_file_pos = 
+  if (this->wave->nBlockAlign < PREFERED_BLOCK_SIZE) {
+    remaining_sample_bytes = PREFERED_BLOCK_SIZE / this->wave->nBlockAlign *
+                             this->wave->nBlockAlign;
+  } else{
+    remaining_sample_bytes = this->wave->nBlockAlign;
+  }
+  current_file_pos =
     this->input->get_current_pos(this->input) - this->data_start;
 
   current_pts = current_file_pos;
@@ -166,7 +174,7 @@ static int demux_wav_send_chunk(demux_plugin_t *this_gen) {
   current_pts /= this->wave->nAvgBytesPerSec;
 
   if (this->seek_flag) {
-    xine_demux_control_newpts(this->stream, current_pts, 0);
+    _x_demux_control_newpts(this->stream, current_pts, BUF_FLAG_SEEK);
     this->seek_flag = 0;
   }
 
@@ -177,9 +185,8 @@ static int demux_wav_send_chunk(demux_plugin_t *this_gen) {
     }
 
     buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-    buf->type = this->audio_type;
-    buf->extra_info->input_pos = current_file_pos;
-    buf->extra_info->input_length = this->data_size;
+    if( this->data_size )
+      buf->extra_info->input_normpos = (int)( (double) current_file_pos * 65535 / this->data_size);
     buf->extra_info->input_time = current_pts / 90;
     buf->pts = current_pts;
 
@@ -196,9 +203,24 @@ static int demux_wav_send_chunk(demux_plugin_t *this_gen) {
       break;
     }
 
+#if 0
+    for(n=0;n<20;n++) {
+      printf("%x ",buf->content[n]);
+    }
+    printf("\n");
+#endif
+
+#if 0
+    for(n=0;n<20;n++) {
+      printf("%x ",buf->content[n]);
+    }
+    printf("\n");
+#endif
+
     if (!remaining_sample_bytes)
       buf->decoder_flags |= BUF_FLAG_FRAME_END;
 
+    buf->type = this->audio_type;
     this->audio_fifo->put (this->audio_fifo, buf);
   }
 
@@ -206,7 +228,6 @@ static int demux_wav_send_chunk(demux_plugin_t *this_gen) {
 }
 
 static void demux_wav_send_headers(demux_plugin_t *this_gen) {
-
   demux_wav_t *this = (demux_wav_t *) this_gen;
   buf_element_t *buf;
 
@@ -216,23 +237,23 @@ static void demux_wav_send_headers(demux_plugin_t *this_gen) {
   this->status = DEMUX_OK;
 
   /* load stream information */
-  this->stream->stream_info[XINE_STREAM_INFO_HAS_VIDEO] = 0;
-  this->stream->stream_info[XINE_STREAM_INFO_HAS_AUDIO] = 1;
-  this->stream->stream_info[XINE_STREAM_INFO_AUDIO_CHANNELS] = 
-    this->wave->nChannels;
-  this->stream->stream_info[XINE_STREAM_INFO_AUDIO_SAMPLERATE] = 
-    this->wave->nSamplesPerSec;
-  this->stream->stream_info[XINE_STREAM_INFO_AUDIO_BITS] = 
-    this->wave->wBitsPerSample;
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_HAS_VIDEO, 0);
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_HAS_AUDIO, 1);
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_CHANNELS,
+                       this->wave->nChannels);
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_SAMPLERATE,
+                       this->wave->nSamplesPerSec);
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_BITS,
+                       this->wave->wBitsPerSample);
 
   /* send start buffers */
-  xine_demux_control_start(this->stream);
+  _x_demux_control_start(this->stream);
 
   /* send init info to decoders */
   if (this->audio_fifo && this->audio_type) {
     buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
     buf->type = this->audio_type;
-    buf->decoder_flags = BUF_FLAG_HEADER;
+    buf->decoder_flags = BUF_FLAG_HEADER|BUF_FLAG_STDHEADER|BUF_FLAG_FRAME_END;
     buf->decoder_info[0] = 0;
     buf->decoder_info[1] = this->wave->nSamplesPerSec;
     buf->decoder_info[2] = this->wave->wBitsPerSample;
@@ -244,9 +265,29 @@ static void demux_wav_send_headers(demux_plugin_t *this_gen) {
 }
 
 static int demux_wav_seek (demux_plugin_t *this_gen,
-                           off_t start_pos, int start_time) {
+                           off_t start_pos, int start_time, int playing) {
 
   demux_wav_t *this = (demux_wav_t *) this_gen;
+  start_pos = (off_t) ( (double) start_pos / 65535 *
+              this->data_size );
+
+  this->seek_flag = 1;
+  this->status = DEMUX_OK;
+  _x_demux_flush_engine (this->stream);
+
+  /* if input is non-seekable, do not proceed with the rest of this
+   * seek function */
+  if (!INPUT_IS_SEEKABLE(this->input))
+    return this->status;
+
+  /* time-based seeking, the start_pos code will then align the blocks
+   * if necessary */
+  if (start_time != 0) {
+    int length = demux_wav_get_stream_length (this_gen);
+    if (length != 0) {
+      start_pos = start_time * this->data_size / length;
+    }
+  }
 
   /* check the boundary offsets */
   if (start_pos <= 0)
@@ -267,20 +308,13 @@ static int demux_wav_seek (demux_plugin_t *this_gen,
     this->input->seek(this->input, start_pos, SEEK_SET);
   }
 
-  this->seek_flag = 1;
-  this->status = DEMUX_OK;
-  xine_demux_flush_engine (this->stream);
-  
-  /* if thread is not running, initialize demuxer */
-  if( !this->stream->demux_thread_running ) {
-  }
-
   return this->status;
 }
 
 static void demux_wav_dispose (demux_plugin_t *this_gen) {
   demux_wav_t *this = (demux_wav_t *) this_gen;
 
+  free(this->wave);
   free(this);
 }
 
@@ -292,7 +326,6 @@ static int demux_wav_get_status (demux_plugin_t *this_gen) {
 
 /* return the approximate length in miliseconds */
 static int demux_wav_get_stream_length (demux_plugin_t *this_gen) {
-
   demux_wav_t *this = (demux_wav_t *) this_gen;
 
   return (int)((int64_t) this->data_size * 1000 / this->wave->nAvgBytesPerSec);
@@ -308,15 +341,10 @@ static int demux_wav_get_optional_data(demux_plugin_t *this_gen,
 }
 
 static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *stream,
-                                    input_plugin_t *input_gen) {
+                                    input_plugin_t *input) {
 
-  input_plugin_t *input = (input_plugin_t *) input_gen;
   demux_wav_t    *this;
-
-  if (! (input->get_capabilities(input) & INPUT_CAP_SEEKABLE)) {
-    printf(_("demux_wav.c: input not seekable, can not handle!\n"));
-    return NULL;
-  }
+  uint32_t	align;
 
   this         = xine_xmalloc (sizeof (demux_wav_t));
   this->stream = stream;
@@ -328,8 +356,6 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   this->demux_plugin.dispose           = demux_wav_dispose;
   this->demux_plugin.get_status        = demux_wav_get_status;
   this->demux_plugin.get_stream_length = demux_wav_get_stream_length;
-  this->demux_plugin.get_video_frame   = NULL;
-  this->demux_plugin.got_video_frame_cb= NULL;
   this->demux_plugin.get_capabilities  = demux_wav_get_capabilities;
   this->demux_plugin.get_optional_data = demux_wav_get_optional_data;
   this->demux_plugin.demux_class       = class_gen;
@@ -338,6 +364,19 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   switch (stream->content_detection_method) {
 
+  case METHOD_BY_EXTENSION: {
+    const char *extensions, *mrl;
+
+    mrl = input->get_mrl (input);
+    extensions = class_gen->get_extensions (class_gen);
+
+    if (!_x_demux_check_extension (mrl, extensions)) {
+      free (this);
+      return NULL;
+    }
+  }
+  /* falling through is intended */
+
   case METHOD_BY_CONTENT:
   case METHOD_EXPLICIT:
 
@@ -348,61 +387,36 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   break;
 
-  case METHOD_BY_EXTENSION: {
-    char *ending, *mrl;
-
-    mrl = input->get_mrl (input);
-
-    ending = strrchr(mrl, '.');
-
-    if (!ending) {
-      free (this);
-      return NULL;
-    }
-
-    if (strncasecmp (ending, ".wav", 4)) {
-      free (this);
-      return NULL;
-    }
-
-    if (!open_wav_file(this)) {
-      free (this);
-      return NULL;
-    }
-
-  }
-
-  break;
-
   default:
     free (this);
     return NULL;
   }
 
-  strncpy (this->last_mrl, input->get_mrl (input), 1024);
-
   /* special block alignment hack so that the demuxer doesn't send
    * packets with individual PCM samples */
   if ((this->wave->nAvgBytesPerSec / this->wave->nBlockAlign) ==
-    this->wave->nSamplesPerSec)
-    this->wave->nBlockAlign = PCM_BLOCK_ALIGN;
+      this->wave->nSamplesPerSec) {
+    align = PCM_BLOCK_ALIGN / this->wave->nBlockAlign;
+    align = align * this->wave->nBlockAlign;
+    this->wave->nBlockAlign = align;
+  }
 
   return &this->demux_plugin;
 }
 
-static char *get_description (demux_class_t *this_gen) {
+static const char *get_description (demux_class_t *this_gen) {
   return "WAV file demux plugin";
 }
 
-static char *get_identifier (demux_class_t *this_gen) {
+static const char *get_identifier (demux_class_t *this_gen) {
   return "WAV";
 }
 
-static char *get_extensions (demux_class_t *this_gen) {
+static const char *get_extensions (demux_class_t *this_gen) {
   return "wav";
 }
 
-static char *get_mimetypes (demux_class_t *this_gen) {
+static const char *get_mimetypes (demux_class_t *this_gen) {
   return "audio/x-wav: wav: WAV audio;"
          "audio/wav: wav: WAV audio;"
          "audio/x-pn-wav: wav: WAV audio;"
@@ -410,19 +424,15 @@ static char *get_mimetypes (demux_class_t *this_gen) {
 }
 
 static void class_dispose (demux_class_t *this_gen) {
-
   demux_wav_class_t *this = (demux_wav_class_t *) this_gen;
 
   free (this);
 }
 
 void *demux_wav_init_plugin (xine_t *xine, void *data) {
-
   demux_wav_class_t     *this;
 
-  this         = xine_xmalloc (sizeof (demux_wav_class_t));
-  this->config = xine->config;
-  this->xine   = xine;
+  this = xine_xmalloc (sizeof (demux_wav_class_t));
 
   this->demux_class.open_plugin     = open_plugin;
   this->demux_class.get_description = get_description;
@@ -433,15 +443,3 @@ void *demux_wav_init_plugin (xine_t *xine, void *data) {
 
   return this;
 }
-
-/*
- * exported plugin catalog entry
- */
-
-#if 0
-plugin_info_t xine_plugin_info[] = {
-  /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_DEMUX, 20, "wav", XINE_VERSION_CODE, NULL, demux_wav_init_plugin },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
-};
-#endif
