@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: input_file.c,v 1.117 2006/10/01 20:14:43 dgp85 Exp $
+ * $Id: input_file.c,v 1.120 2007/01/19 01:05:25 dgp85 Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -299,7 +299,7 @@ static int is_a_dir(char *filepathname) {
   return (S_ISDIR(pstat.st_mode));
 }
 
-static char* file_plugin_get_mrl (input_plugin_t *this_gen) {
+static const char* file_plugin_get_mrl (input_plugin_t *this_gen) {
   file_input_plugin_t *this = (file_input_plugin_t *) this_gen;
 
   return this->mrl;
@@ -659,7 +659,7 @@ static off_t get_file_size(char *filepathname, char *origin) {
   return pstat.st_size;
 }
 
-static char *file_class_get_description (input_class_t *this_gen) {
+static const char *file_class_get_description (input_class_t *this_gen) {
   return _("file input plugin");
 }
 
@@ -1027,8 +1027,8 @@ static void *init_plugin (xine_t *xine, void *data) {
     if(getcwd(current_dir, sizeof(current_dir)) == NULL)
       strcpy(current_dir, ".");
 
-    this->origin_path = config->register_string(config, "media.files.origin_path",
-						current_dir, 
+    this->origin_path = config->register_filename(config, "media.files.origin_path",
+						current_dir, XINE_CONFIG_STRING_IS_DIRECTORY_NAME,
 						_("file browsing start location"),
 						_("The browser to select the file to play will "
 						  "start at this location."),
