@@ -50,10 +50,6 @@
 
 #define BUFSIZE 1025
 
-#if !defined(NDELAY) && defined(O_NDELAY)
-#define FNDELAY O_NDELAY
-#endif
-
 typedef struct {
 
   input_class_t     input_class;
@@ -106,7 +102,7 @@ static buf_element_t *rtsp_plugin_read_block (input_plugin_t *this_gen,
   buf->content = buf->mem;
   buf->type = BUF_DEMUX_BLOCK;
   
-  total_bytes = rtsp_plugin_read (this_gen, buf->content, todo);
+  total_bytes = rtsp_plugin_read (this_gen, (char*)buf->content, todo);
 
   if (total_bytes != todo) {
     buf->free_buffer (buf);
@@ -138,6 +134,18 @@ static off_t rtsp_plugin_seek (input_plugin_t *this_gen, off_t offset, int origi
   return this->curpos;
 }
 
+static off_t rtsp_plugin_seek_time (input_plugin_t *this_gen, int time_offset, int origin) {
+
+  rtsp_input_plugin_t *this = (rtsp_input_plugin_t *) this_gen;
+
+  lprintf ("seek_time %d msec, origin %d\n", time_offset, origin);
+  
+  if (origin == SEEK_SET)
+    rtsp_session_set_start_time (this->rtsp, time_offset);
+    
+  return this->curpos;
+}
+
 static off_t rtsp_plugin_get_length (input_plugin_t *this_gen) {
 
   /*
@@ -188,7 +196,7 @@ static void rtsp_plugin_dispose (input_plugin_t *this_gen) {
   free (this);
 }
 
-static char* rtsp_plugin_get_mrl (input_plugin_t *this_gen) {
+static const char* rtsp_plugin_get_mrl (input_plugin_t *this_gen) {
   rtsp_input_plugin_t *this = (rtsp_input_plugin_t *) this_gen;
 
   return this->public_mrl;
@@ -216,7 +224,7 @@ static int rtsp_plugin_open (input_plugin_t *this_gen) {
 
   lprintf ("trying to open '%s'\n", this->mrl);
 
-  rtsp = rtsp_session_start(this->stream,this->mrl);
+  rtsp = rtsp_session_start(this->stream, this->mrl);
 
   if (!rtsp) {
     lprintf ("returning null.\n");
@@ -230,22 +238,19 @@ static int rtsp_plugin_open (input_plugin_t *this_gen) {
 }
 
 static input_plugin_t *rtsp_class_get_instance (input_class_t *cls_gen, xine_stream_t *stream, 
-				    const char *data) {
+				    const char *mrl) {
 
   /* rtsp_input_class_t  *cls = (rtsp_input_class_t *) cls_gen; */
   rtsp_input_plugin_t *this;
-  char                *mrl = strdup(data);
 
-  if (strncasecmp (mrl, "rtsp://", 6)) {
-    free (mrl);
+  if (strncasecmp (mrl, "rtsp://", 6))
     return NULL;
-  }
 
   this = (rtsp_input_plugin_t *) xine_xmalloc (sizeof (rtsp_input_plugin_t));
 
   this->stream  = stream;
   this->rtsp    = NULL;
-  this->mrl     = mrl;
+  this->mrl     = strdup (mrl);
   /* since we handle only real streams yet, we can savely add
    * an .rm extention to force handling by demux_real.
    */
@@ -259,6 +264,7 @@ static input_plugin_t *rtsp_class_get_instance (input_class_t *cls_gen, xine_str
   this->input_plugin.read              = rtsp_plugin_read;
   this->input_plugin.read_block        = rtsp_plugin_read_block;
   this->input_plugin.seek              = rtsp_plugin_seek;
+  this->input_plugin.seek_time         = rtsp_plugin_seek_time;
   this->input_plugin.get_current_pos   = rtsp_plugin_get_current_pos;
   this->input_plugin.get_length        = rtsp_plugin_get_length;
   this->input_plugin.get_blocksize     = rtsp_plugin_get_blocksize;
@@ -274,7 +280,7 @@ static input_plugin_t *rtsp_class_get_instance (input_class_t *cls_gen, xine_str
  * rtsp input plugin class stuff
  */
 
-static char *rtsp_class_get_description (input_class_t *this_gen) {
+static const char *rtsp_class_get_description (input_class_t *this_gen) {
   return _("rtsp streaming input plugin");
 }
 
@@ -311,9 +317,9 @@ static void *init_class (xine_t *xine, void *data) {
  * exported plugin catalog entry
  */
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_INPUT, 16, "rtsp", XINE_VERSION_CODE, NULL, init_class },
+  { PLUGIN_INPUT, 17, "rtsp", XINE_VERSION_CODE, NULL, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
 
