@@ -30,12 +30,6 @@
 #include "mpeg2_internal.h"
 #include "attributes.h"
 
-extern mpeg2_mc_t mpeg2_mc;
-extern void (* mpeg2_idct_copy) (int16_t * block, uint8_t * dest, int stride);
-extern void (* mpeg2_idct_add) (int16_t * block, uint8_t * dest, int stride);
-extern void (* mpeg2_cpu_state_save) (cpu_state_t * state);
-extern void (* mpeg2_cpu_state_restore) (cpu_state_t * state);
-
 #include "vlc.h"
 
 static int non_linear_quantizer_scale [] = {
@@ -51,7 +45,7 @@ static inline int get_macroblock_modes (picture_t * picture)
 #define bits (picture->bitstream_bits)
 #define bit_ptr (picture->bitstream_ptr)
     int macroblock_modes;
-    MBtab * tab;
+    const MBtab * tab;
 
     switch (picture->picture_coding_type) {
     case I_TYPE:
@@ -166,7 +160,7 @@ static inline int get_motion_delta (picture_t * picture, int f_code)
 
     int delta;
     int sign;
-    MVtab * tab;
+    const MVtab * tab;
 
     if (bit_buf & 0x80000000) {
 	DUMPBITS (bit_buf, bits, 1);
@@ -211,7 +205,7 @@ static inline int get_motion_delta (picture_t * picture, int f_code)
 #undef bit_ptr
 }
 
-static inline int bound_motion_vector (int vector, int f_code)
+static inline int bound_motion_vector (int vec, int f_code)
 {
 #if 1
     unsigned int limit;
@@ -219,11 +213,11 @@ static inline int bound_motion_vector (int vector, int f_code)
 
     limit = 16 << f_code;
 
-    if ((unsigned int)(vector + limit) < 2 * limit)
-	return vector;
+    if ((unsigned int)(vec + limit) < 2 * limit)
+	return vec;
     else {
-	sign = ((int32_t)vector) >> 31;
-	return vector - ((2 * limit) ^ sign) + sign;
+	sign = ((int32_t)vec) >> 31;
+	return vec - ((2 * limit) ^ sign) + sign;
     }
 #else
     return ((int32_t)vector << (27 - f_code)) >> (27 - f_code);
@@ -236,7 +230,7 @@ static inline int get_dmv (picture_t * picture)
 #define bits (picture->bitstream_bits)
 #define bit_ptr (picture->bitstream_ptr)
 
-    DMVtab * tab;
+    const DMVtab * tab;
 
     tab = DMV_2 + UBITS (bit_buf, 2);
     DUMPBITS (bit_buf, bits, tab->len);
@@ -252,7 +246,7 @@ static inline int get_coded_block_pattern (picture_t * picture)
 #define bits (picture->bitstream_bits)
 #define bit_ptr (picture->bitstream_ptr)
 
-    CBPtab * tab;
+    const CBPtab * tab;
 
     NEEDBITS (bit_buf, bits, bit_ptr);
 
@@ -279,7 +273,7 @@ static inline int get_luma_dc_dct_diff (picture_t * picture)
 #define bit_buf (picture->bitstream_buf)
 #define bits (picture->bitstream_bits)
 #define bit_ptr (picture->bitstream_ptr)
-    DCtab * tab;
+    const DCtab * tab;
     int size;
     int dc_diff;
 
@@ -316,7 +310,7 @@ static inline int get_chroma_dc_dct_diff (picture_t * picture)
 #define bit_buf (picture->bitstream_buf)
 #define bits (picture->bitstream_bits)
 #define bit_ptr (picture->bitstream_ptr)
-    DCtab * tab;
+    const DCtab * tab;
     int size;
     int dc_diff;
 
@@ -363,7 +357,7 @@ static void get_intra_block_B14 (picture_t * picture)
     uint8_t * quant_matrix = picture->intra_quantizer_matrix;
     int quantizer_scale = picture->quantizer_scale;
     int mismatch;
-    DCTtab * tab;
+    const DCTtab * tab;
     uint32_t bit_buf;
     int bits;
     uint8_t * bit_ptr;
@@ -477,7 +471,7 @@ static void get_intra_block_B15 (picture_t * picture)
     uint8_t * quant_matrix = picture->intra_quantizer_matrix;
     int quantizer_scale = picture->quantizer_scale;
     int mismatch;
-    DCTtab * tab;
+    const DCTtab * tab;
     uint32_t bit_buf;
     int bits;
     uint8_t * bit_ptr;
@@ -590,7 +584,7 @@ static void get_non_intra_block (picture_t * picture)
     uint8_t * quant_matrix = picture->non_intra_quantizer_matrix;
     int quantizer_scale = picture->quantizer_scale;
     int mismatch;
-    DCTtab * tab;
+    const DCTtab * tab;
     uint32_t bit_buf;
     int bits;
     uint8_t * bit_ptr;
@@ -712,7 +706,7 @@ static void get_mpeg1_intra_block (picture_t * picture)
     uint8_t * scan = picture->scan;
     uint8_t * quant_matrix = picture->intra_quantizer_matrix;
     int quantizer_scale = picture->quantizer_scale;
-    DCTtab * tab;
+    const DCTtab * tab;
     uint32_t bit_buf;
     int bits;
     uint8_t * bit_ptr;
@@ -831,7 +825,7 @@ static void get_mpeg1_non_intra_block (picture_t * picture)
     uint8_t * scan = picture->scan;
     uint8_t * quant_matrix = picture->non_intra_quantizer_matrix;
     int quantizer_scale = picture->quantizer_scale;
-    DCTtab * tab;
+    const DCTtab * tab;
     uint32_t bit_buf;
     int bits;
     uint8_t * bit_ptr;
@@ -1438,29 +1432,27 @@ do {								\
 		  mpeg2_mc.avg : mpeg2_mc.put));		\
 } while (0)
 
-#define NEXT_MACROBLOCK							\
-do {									\
-    picture->offset += 16;						\
-    if (picture->offset == picture->coded_picture_width) {		\
-	do { /* just so we can use the break statement */		\
-	    if (picture->current_frame->copy) {				\
-		picture->current_frame->copy (picture->current_frame,	\
-					      picture->dest);		\
-		if (picture->picture_coding_type == B_TYPE)		\
-		    break;						\
-	    }								\
-	    picture->dest[0] += 16 * picture->pitches[0];		\
-	    picture->dest[1] += 8 * picture->pitches[1];		\
-	    picture->dest[2] += 8 * picture->pitches[2];		\
-	} while (0);							\
-	picture->v_offset += 16;					\
-	if (picture->v_offset > picture->limit_y) {			\
-	    if (mpeg2_cpu_state_restore)				\
-		mpeg2_cpu_state_restore (&cpu_state);			\
-	    return;							\
-	}								\
-	picture->offset = 0;						\
-    }									\
+#define NEXT_MACROBLOCK							    \
+do {									    \
+    picture->offset += 16;						    \
+    if (picture->offset == picture->coded_picture_width) {		    \
+	do { /* just so we can use the break statement */		    \
+	    if (picture->current_frame->proc_slice) {			    \
+		picture->current_frame->proc_slice (picture->current_frame, \
+						    picture->dest);	    \
+	    }								    \
+	    picture->dest[0] += 16 * picture->pitches[0];		    \
+	    picture->dest[1] += 8 * picture->pitches[1];		    \
+	    picture->dest[2] += 8 * picture->pitches[2];		    \
+	} while (0);							    \
+	picture->v_offset += 16;					    \
+	if (picture->v_offset > picture->limit_y) {			    \
+	    if (mpeg2_cpu_state_restore)				    \
+		mpeg2_cpu_state_restore (&cpu_state);			    \
+	    return;							    \
+	}								    \
+	picture->offset = 0;						    \
+    }									    \
 } while (0)
 
 static inline int slice_init (picture_t * picture, int code)
@@ -1471,7 +1463,7 @@ static inline int slice_init (picture_t * picture, int code)
     int offset, height;
     struct vo_frame_s * forward_reference_frame;
     struct vo_frame_s * backward_reference_frame;
-    MBAtab * mba;
+    const MBAtab * mba;
 
     offset = picture->picture_structure == BOTTOM_FIELD;
     picture->pitches[0] = picture->current_frame->pitches[0];
@@ -1542,9 +1534,7 @@ static inline int slice_init (picture_t * picture, int code)
 
     picture->v_offset = (code - 1) * 16;
     offset = (code - 1);
-    if (picture->current_frame->copy && picture->picture_coding_type == B_TYPE)
-	offset = 0;
-    else if (picture->picture_structure != FRAME_PICTURE)
+    if (picture->picture_structure != FRAME_PICTURE)
 	offset = 2 * offset;
 
     picture->dest[0] = picture->current_frame->base[0] + picture->pitches[0] * offset * 16;
@@ -1609,7 +1599,7 @@ static inline int slice_init (picture_t * picture, int code)
 
     while (picture->offset - picture->coded_picture_width >= 0) {
 	picture->offset -= picture->coded_picture_width;
-	if ((picture->current_frame->copy == NULL) ||
+	if ((picture->current_frame->proc_slice == NULL) ||
 	    (picture->picture_coding_type != B_TYPE)) {
 	    picture->dest[0] += 16 * picture->pitches[0];
 	    picture->dest[1] += 8 * picture->pitches[1];
@@ -1644,7 +1634,7 @@ void mpeg2_slice (picture_t * picture, int code, uint8_t * buffer)
     while (1) {
 	int macroblock_modes;
 	int mba_inc;
-	MBAtab * mba;
+	const MBAtab * mba;
 
 	NEEDBITS (bit_buf, bits, bit_ptr);
 
