@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: video_out.c,v 1.214.2.1 2005/04/20 17:21:17 mroi Exp $
+ * $Id: video_out.c,v 1.227 2007/01/16 16:00:26 miguelfreitas Exp $
  *
  * frame allocation / queuing / scheduling / output functions
  */
@@ -59,6 +59,12 @@
 #define FIRST_FRAME_POLL_DELAY   3000
 #define FIRST_FRAME_MAX_POLL       10    /* poll n times at most */
 
+/* experimental optimization: try to allocate frames from free queue 
+ * in the same format as requested (avoid unnecessary free/alloc in
+ * vo driver). up to 25% less cpu load using deinterlace with film mode.
+ */ 
+#define EXPERIMENTAL_FRAME_QUEUE_OPTIMIZATION 1
+
 static vo_frame_t * crop_frame( xine_video_port_t *this_gen, vo_frame_t *img );
 
 typedef struct {
@@ -174,33 +180,85 @@ static void vo_append_to_img_buf_queue (img_buf_fifo_t *queue,
   pthread_mutex_unlock (&queue->mutex);
 }
 
-static vo_frame_t *vo_remove_from_img_buf_queue_int (img_buf_fifo_t *queue, int blocking) {
-  vo_frame_t *img;
+static vo_frame_t *vo_remove_from_img_buf_queue_int (img_buf_fifo_t *queue, int blocking,
+                                                     uint32_t width, uint32_t height,
+                                                     double ratio, int format,
+                                                     int flags) {
+  vo_frame_t *img = NULL;
+  vo_frame_t *previous = NULL;
 
-  while (!queue->first || queue->locked_for_read) {
-    if (blocking)
-      pthread_cond_wait (&queue->not_empty, &queue->mutex);
-    else {
-      struct timeval tv;
-      struct timespec ts;
-      gettimeofday(&tv, NULL);
-      ts.tv_sec  = tv.tv_sec + 1;
-      ts.tv_nsec = tv.tv_usec * 1000;
-      if (pthread_cond_timedwait (&queue->not_empty, &queue->mutex, &ts) != 0)
-        return NULL;
+  while (!img || queue->locked_for_read) {
+
+    img = (queue->locked_for_read) ? NULL : queue->first;
+
+#if EXPERIMENTAL_FRAME_QUEUE_OPTIMIZATION
+    if (img) {
+      /* try to obtain a frame with the same format first.
+       * doing so may avoid unnecessary alloc/free's at the vo
+       * driver, specially when using post plugins that change
+       * format like the tvtime deinterlacer does.
+       */
+      int i = 0;
+      while( img && width && height &&
+            (img->width != width || img->height != height ||
+            img->ratio != ratio || img->format != format) ) {
+        previous = img;
+        img = img->next;
+        i++;
+      }
+      
+      if( width && height ) {
+        if( !img ) {
+          if( queue->num_buffers == 1 && !blocking) {
+            /* non-blocking and only a single frame on fifo with different
+             * format -> ignore it (give another chance of a frame format hit)
+             */
+            lprintf("frame format mismatch - will wait another frame\n");
+          } else {
+            /* we have at least 2 frames on fifo but they don't match ->
+             * give up. return whatever we got.
+             */
+            img = queue->first;
+            lprintf("frame format miss (%d/%d)\n", i, queue->num_buffers);
+          }
+        } else {
+          /* good: format match! */
+          lprintf("frame format hit (%d/%d)\n", i, queue->num_buffers);
+        }
+      }
+    }
+#endif
+
+    if(!img) {
+      if (blocking)
+        pthread_cond_wait (&queue->not_empty, &queue->mutex);
+      else {
+        struct timeval tv;
+        struct timespec ts;
+        gettimeofday(&tv, NULL);
+        ts.tv_sec  = tv.tv_sec + 1;
+        ts.tv_nsec = tv.tv_usec * 1000;
+        if (pthread_cond_timedwait (&queue->not_empty, &queue->mutex, &ts) != 0)
+          return NULL;
+      }
     }
   }
 
-  img = queue->first;
-
   if (img) {
-    queue->first = img->next;
+
+    if( img == queue->first ) {
+      queue->first = img->next;
+    } else {
+      previous->next = img->next;
+      if( img == queue->last )
+        queue->last = previous;
+    }
+    
     img->next = NULL;
     if (!queue->first) {
       queue->last = NULL;
       queue->num_buffers = 0;
-    }
-    else {
+    } else {
       queue->num_buffers--;
     }
   }
@@ -212,17 +270,20 @@ static vo_frame_t *vo_remove_from_img_buf_queue (img_buf_fifo_t *queue) {
   vo_frame_t *img;
 
   pthread_mutex_lock (&queue->mutex);
-  img = vo_remove_from_img_buf_queue_int(queue, 1);
+  img = vo_remove_from_img_buf_queue_int(queue, 1, 0, 0, 0, 0, 0);
   pthread_mutex_unlock (&queue->mutex);
 
   return img;
 }
 
-static vo_frame_t *vo_remove_from_img_buf_queue_nonblock (img_buf_fifo_t *queue) {
+static vo_frame_t *vo_remove_from_img_buf_queue_nonblock (img_buf_fifo_t *queue,
+                                                          uint32_t width, uint32_t height,
+                                                          double ratio, int format,
+                                                          int flags) {
   vo_frame_t *img;
 
   pthread_mutex_lock (&queue->mutex);
-  img = vo_remove_from_img_buf_queue_int(queue, 0);
+  img = vo_remove_from_img_buf_queue_int(queue, 0, width, height, ratio, format, flags);
   pthread_mutex_unlock (&queue->mutex);
 
   return img;
@@ -312,7 +373,8 @@ static vo_frame_t *vo_get_frame (xine_video_port_t *this_gen,
 
   lprintf ("get_frame (%d x %d)\n", width, height);
 
-  while (!(img = vo_remove_from_img_buf_queue_nonblock (this->free_img_buf_queue)))
+  while (!(img = vo_remove_from_img_buf_queue_nonblock (this->free_img_buf_queue,
+                 width, height, ratio, format, flags)))
     if (this->xine->port_ticket->ticket_revoked)
       this->xine->port_ticket->renew(this->xine->port_ticket, 1);
 
@@ -338,6 +400,8 @@ static vo_frame_t *vo_get_frame (xine_video_port_t *this_gen,
   img->crop_right     = 0;
   img->crop_top       = 0;
   img->crop_bottom    = 0;
+  img->overlay_offset_x = 0;
+  img->overlay_offset_y = 0;
   img->stream         = NULL;
 
   _x_extra_info_reset ( img->extra_info );
@@ -438,6 +502,7 @@ static int vo_frame_draw (vo_frame_t *img, xine_stream_t *stream) {
   if (!img->bad_frame) {
   
     int img_already_locked = 0;
+    xine_list_iterator_t ite;
     
     /* add cropping requested by frontend */
     img->crop_left   += this->crop_left;
@@ -451,6 +516,8 @@ static int vo_frame_draw (vo_frame_t *img, xine_stream_t *stream) {
         (this->grab_only ||
          !(this->driver->get_capabilities (this->driver) & VO_CAP_CROP)) ) {
       if (img->format == XINE_IMGFMT_YV12 || img->format == XINE_IMGFMT_YUY2) {
+        img->overlay_offset_x -= img->crop_left;
+        img->overlay_offset_y -= img->crop_top;
         img = crop_frame( img->port, img );
         img_already_locked = 1;
       } else {
@@ -477,8 +544,9 @@ static int vo_frame_draw (vo_frame_t *img, xine_stream_t *stream) {
      */
     img->is_first = 0;
     pthread_mutex_lock(&this->streams_lock);
-    for (stream = xine_list_first_content(this->streams); stream;
-         stream = xine_list_next_content(this->streams)) {
+    for (ite = xine_list_front(this->streams); ite;
+         ite = xine_list_next(this->streams, ite)) {
+      stream = xine_list_get_value(this->streams, ite);
       if (stream == XINE_ANON_STREAM) continue;
       pthread_mutex_lock (&stream->first_frame_lock);
       if (stream->first_frame_flag == 2) {
@@ -517,6 +585,7 @@ static int vo_frame_draw (vo_frame_t *img, xine_stream_t *stream) {
 
   if ((this->num_frames_delivered % 200) == 0 && this->num_frames_delivered) {
     int send_event;
+    xine_list_iterator_t ite;
 
     if( (100 * this->num_frames_skipped / this->num_frames_delivered) >
          this->warn_skipped_threshold ||
@@ -534,8 +603,9 @@ static int vo_frame_draw (vo_frame_t *img, xine_stream_t *stream) {
     this->warn_threshold_event_sent += send_event;
 
     pthread_mutex_lock(&this->streams_lock);
-    for (stream = xine_list_first_content(this->streams); stream;
-         stream = xine_list_next_content(this->streams)) {
+    for (ite = xine_list_front(this->streams); ite;
+         ite = xine_list_next(this->streams, ite)) {
+      stream = xine_list_get_value(this->streams, ite);
       if (stream == XINE_ANON_STREAM) continue;
       _x_stream_info_set(stream, XINE_STREAM_INFO_SKIPPED_FRAMES, 
 			 1000 * this->num_frames_skipped / this->num_frames_delivered);
@@ -626,6 +696,8 @@ static vo_frame_t * duplicate_frame( vos_t *this, vo_frame_t *img ) {
   dupl->crop_right     = img->crop_right;
   dupl->crop_top       = img->crop_top;
   dupl->crop_bottom    = img->crop_bottom;
+  dupl->overlay_offset_x = img->overlay_offset_x;
+  dupl->overlay_offset_y = img->overlay_offset_y;
   
   this->driver->update_frame_format (this->driver, dupl, dupl->width, dupl->height, 
 				     dupl->ratio, dupl->format, dupl->flags);
@@ -741,7 +813,7 @@ static void expire_frames (vos_t *this, int64_t cur_vpts) {
         this->num_frames_discarded++;
       }
       
-      img = vo_remove_from_img_buf_queue_int (this->display_img_buf_queue, 1);
+      img = vo_remove_from_img_buf_queue_int (this->display_img_buf_queue, 1, 0, 0, 0, 0, 0);
   
       if (img->stream) {
 	pthread_mutex_lock( &img->stream->current_extra_info_lock );
@@ -889,7 +961,7 @@ static vo_frame_t *get_next_frame (vos_t *this, int64_t cur_vpts,
      * remove frame from display queue and show it
      */
     
-    img = vo_remove_from_img_buf_queue_int (this->display_img_buf_queue, 1);
+    img = vo_remove_from_img_buf_queue_int (this->display_img_buf_queue, 1, 0, 0, 0, 0, 0);
     pthread_mutex_unlock(&this->display_img_buf_queue->mutex);
 
     return img;
@@ -899,6 +971,7 @@ static vo_frame_t *get_next_frame (vos_t *this, int64_t cur_vpts,
 static void overlay_and_display_frame (vos_t *this, 
 				       vo_frame_t *img, int64_t vpts) {
   xine_stream_t *stream;
+  xine_list_iterator_t ite;
 
   lprintf ("displaying image with vpts = %" PRId64 "\n", img->vpts);
 
@@ -938,8 +1011,9 @@ static void overlay_and_display_frame (vos_t *this,
    */
   if( this->last_frame->is_first ) {
     pthread_mutex_lock(&this->streams_lock);
-    for (stream = xine_list_first_content(this->streams); stream;
-         stream = xine_list_next_content(this->streams)) {
+    for (ite = xine_list_front(this->streams); ite;
+         ite = xine_list_next(this->streams, ite)) {
+      stream = xine_list_get_value(this->streams, ite);
       if (stream == XINE_ANON_STREAM) continue;
       pthread_mutex_lock (&stream->first_frame_lock);
       if (stream->first_frame_flag) {
@@ -1081,11 +1155,12 @@ static void *video_out_loop (void *this_gen) {
 
     diff = vpts - this->last_delivery_pts;
     if (diff > 30000 && !this->display_img_buf_queue->first) {
-      xine_stream_t *stream;
+      xine_list_iterator_t ite;
       
       pthread_mutex_lock(&this->streams_lock);
-      for (stream = xine_list_first_content(this->streams); stream;
-           stream = xine_list_next_content(this->streams)) {
+      for (ite = xine_list_front(this->streams); ite;
+           ite = xine_list_next(this->streams, ite)) {
+	xine_stream_t *stream = xine_list_get_value(this->streams, ite);
 	if (stream == XINE_ANON_STREAM) continue;
         if (stream->video_decoder_plugin && stream->video_fifo) {
           buf_element_t *buf;
@@ -1120,8 +1195,8 @@ static void *video_out_loop (void *this_gen) {
       if (this->clock->speed == XINE_SPEED_PAUSE)
         paused_loop (this, vpts);
 
-      if (next_frame_vpts) {
-        usec_to_sleep = (next_frame_vpts - vpts) * 100 / 9;
+      if (next_frame_vpts && this->clock->speed > 0) {
+        usec_to_sleep = (next_frame_vpts - vpts) * 100 * XINE_FINE_SPEED_NORMAL / (9 * this->clock->speed);
       } else {
         /* we don't know when the next frame is due, only wait a little */
         usec_to_sleep = 1000;
@@ -1155,7 +1230,7 @@ static void *video_out_loop (void *this_gen) {
   img = this->display_img_buf_queue->first;
   while (img) {
 
-    img = vo_remove_from_img_buf_queue_int (this->display_img_buf_queue, 1);
+    img = vo_remove_from_img_buf_queue_int (this->display_img_buf_queue, 1, 0, 0, 0, 0, 0);
     vo_frame_dec_lock( img );
 
     img = this->display_img_buf_queue->first;
@@ -1187,12 +1262,12 @@ int xine_get_next_video_frame (xine_video_port_t *this_gen,
   xine_stream_t *stream = NULL;
 
   while (!img || !stream) {
-    stream = xine_list_first_content(this->streams);
+    xine_list_iterator_t ite = xine_list_front(this->streams);
+    stream = xine_list_get_value(this->streams, ite);
     if (!stream) {
       xine_usec_sleep (5000);
       continue;
     }
-
     
     /* FIXME: ugly, use conditions and locks instead? */
     
@@ -1213,7 +1288,7 @@ int xine_get_next_video_frame (xine_video_port_t *this_gen,
    * remove frame from display queue and show it
    */
     
-  img = vo_remove_from_img_buf_queue_int (this->display_img_buf_queue, 1);
+  img = vo_remove_from_img_buf_queue_int (this->display_img_buf_queue, 1, 0, 0, 0, 0, 0);
   pthread_mutex_unlock(&this->display_img_buf_queue->mutex);
 
   frame->vpts         = img->vpts;
@@ -1259,14 +1334,14 @@ static void vo_open (xine_video_port_t *this_gen, xine_stream_t *stream) {
     /* enable overlays if our new stream might want to show some */
     this->overlay_enabled = 1;
   pthread_mutex_lock(&this->streams_lock);
-  xine_list_append_content(this->streams, stream);
+  xine_list_push_back(this->streams, stream);
   pthread_mutex_unlock(&this->streams_lock);
 }
 
 static void vo_close (xine_video_port_t *this_gen, xine_stream_t *stream) {
 
   vos_t      *this = (vos_t *) this_gen;
-  xine_stream_t *cur;
+  xine_list_iterator_t ite;
 
   /* this will make sure all hide events were processed */
   if (this->overlay_source)
@@ -1276,19 +1351,20 @@ static void vo_close (xine_video_port_t *this_gen, xine_stream_t *stream) {
   
   /* unregister stream */
   pthread_mutex_lock(&this->streams_lock);
-  for (cur = xine_list_first_content(this->streams); cur;
-       cur = xine_list_next_content(this->streams))
+  for (ite = xine_list_front(this->streams); ite;
+       ite = xine_list_next(this->streams, ite)) {
+    xine_stream_t *cur = xine_list_get_value(this->streams, ite);
     if (cur == stream) {
-      xine_list_delete_current(this->streams);
+      xine_list_remove(this->streams, ite);
       break;
     }
+  }
   pthread_mutex_unlock(&this->streams_lock);
 }
 
 
 static int vo_get_property (xine_video_port_t *this_gen, int property) {
   vos_t *this = (vos_t *) this_gen;
-  xine_stream_t *cur;
   int ret;
 
   switch (property) {
@@ -1301,11 +1377,8 @@ static int vo_get_property (xine_video_port_t *this_gen, int property) {
     break;
   
   case VO_PROP_NUM_STREAMS:
-    ret = 0;
     pthread_mutex_lock(&this->streams_lock);
-    for (cur = xine_list_first_content(this->streams); cur;
-         cur = xine_list_next_content(this->streams))
-      ret++;
+    ret = xine_list_size(this->streams);
     pthread_mutex_unlock(&this->streams_lock);
     break;
   
@@ -1382,7 +1455,7 @@ static int vo_set_property (xine_video_port_t *this_gen, int property, int value
   
         lprintf ("flushing out frame\n");
   
-        img = vo_remove_from_img_buf_queue_int (this->display_img_buf_queue, 1);
+        img = vo_remove_from_img_buf_queue_int (this->display_img_buf_queue, 1, 0, 0, 0, 0, 0);
   
         vo_frame_dec_lock (img);
       }
@@ -1456,12 +1529,13 @@ static int vo_status (xine_video_port_t *this_gen, xine_stream_t *stream,
                       int *width, int *height, int64_t *img_duration) {
 
   vos_t      *this = (vos_t *) this_gen;
-  xine_stream_t *cur;
+  xine_list_iterator_t ite;
   int ret = 0;
 
   pthread_mutex_lock(&this->streams_lock);
-  for (cur = xine_list_first_content(this->streams); cur;
-       cur = xine_list_next_content(this->streams))
+  for (ite = xine_list_front(this->streams); ite;
+       ite = xine_list_next(this->streams, ite)) {
+    xine_stream_t *cur = xine_list_get_value(this->streams, ite);
     if (cur == stream || !stream) {
       *width = this->current_width;
       *height = this->current_height;
@@ -1469,6 +1543,7 @@ static int vo_status (xine_video_port_t *this_gen, xine_stream_t *stream,
       ret = !!stream; /* return false for a NULL stream, true otherwise */
       break;
     }
+  }
   pthread_mutex_unlock(&this->streams_lock);
   
   return ret;
@@ -1516,7 +1591,7 @@ static void vo_exit (xine_video_port_t *this_gen) {
     this->overlay_source->dispose (this->overlay_source);
   }
   
-  xine_list_free(this->streams);
+  xine_list_delete(this->streams);
   pthread_mutex_destroy(&this->streams_lock);
 
   free (this->free_img_buf_queue);
@@ -1547,10 +1622,12 @@ static void vo_enable_overlay (xine_video_port_t *this_gen, int overlay_enabled)
     this->overlay_enabled = 1;
   } else {
     /* ... but we only actually DISable, if all associated streams have SPU off */
-    xine_stream_t *stream;
+    xine_list_iterator_t ite;
+
     pthread_mutex_lock(&this->streams_lock);
-    for (stream = xine_list_first_content(this->streams) ; stream ;
-         stream = xine_list_next_content(this->streams)) {
+    for (ite = xine_list_front(this->streams); ite;
+         ite = xine_list_next(this->streams, ite)) {
+      xine_stream_t *stream = xine_list_get_value(this->streams, ite);
       if (stream == XINE_ANON_STREAM || stream->spu_channel_user > -2) {
 	pthread_mutex_unlock(&this->streams_lock);
 	return;
@@ -1604,6 +1681,8 @@ static vo_frame_t * crop_frame( xine_video_port_t *this_gen, vo_frame_t *img ) {
   dupl->progressive_frame  = img->progressive_frame;
   dupl->repeat_first_field = img->repeat_first_field;
   dupl->top_field_first    = img->top_field_first;
+  dupl->overlay_offset_x   = img->overlay_offset_x;
+  dupl->overlay_offset_y   = img->overlay_offset_y;
   
   switch (img->format) {
   case XINE_IMGFMT_YV12:
@@ -1704,11 +1783,23 @@ xine_video_port_t *_x_vo_new_port (xine_t *xine, vo_driver_t *driver, int grabon
   this->frame_drop_limit      = 3;
   this->frame_drop_cpt        = 0;
 
-  num_frame_buffers = driver->get_property (driver, VO_PROP_MAX_NUM_FRAMES);
+  /* default number of video frames from config */
+  num_frame_buffers = xine->config->register_num (xine->config,
+                                                  "engine.buffers.video_num_frames",
+                                                  NUM_FRAME_BUFFERS, /* default */
+                                                  _("default number of video frames"),
+						  _("The default number of video frames to request "
+						    "from xine video out driver. Some drivers will "
+						    "override this setting with their own values."),
+                                                    20, NULL, NULL);
 
-  if (!num_frame_buffers)
-    num_frame_buffers = NUM_FRAME_BUFFERS; /* default */
-  else if (num_frame_buffers<5) 
+  /* check driver's limit and use the smaller value */
+  i = driver->get_property (driver, VO_PROP_MAX_NUM_FRAMES);
+  if (i && i < num_frame_buffers)
+    num_frame_buffers = i;
+
+  /* we need at least 5 frames */
+  if (num_frame_buffers<5) 
     num_frame_buffers = 5;
 
   this->extra_info_base = calloc (num_frame_buffers,
