@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2004 the xine project
+ * Copyright (C) 2000-2006 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine.c,v 1.306.2.4 2005/07/21 02:57:52 miguelfreitas Exp $
+ * $Id: xine.c,v 1.340 2006/12/22 16:38:15 klan Exp $
  */
 
 /*
@@ -36,6 +36,7 @@
 #include <stdlib.h>
 #include <pthread.h>
 #include <stdarg.h>
+#include <stdio.h>
 #if defined (__linux__)
 #include <endian.h>
 #elif defined (__FreeBSD__)
@@ -77,6 +78,10 @@
 #endif /* WIN32 */
 
 
+static void mutex_cleanup (void *mutex) {
+  pthread_mutex_unlock ((pthread_mutex_t *) mutex);
+}
+
 void _x_handle_stream_end (xine_stream_t *stream, int non_user) {
 
   if (stream->status == XINE_STATUS_QUIT)
@@ -88,7 +93,7 @@ void _x_handle_stream_end (xine_stream_t *stream, int non_user) {
      * if they have called xine_stop explicitly, so only send
      * it if stream playback finished because of stream end reached
      */
-
+    
     xine_event_t event;
 
     event.data_length = 0;
@@ -232,6 +237,7 @@ static xine_ticket_t *ticket_init(void) {
   pthread_mutex_init(&port_ticket->lock, NULL);
   pthread_mutex_init(&port_ticket->revoke_lock, NULL);
   pthread_cond_init(&port_ticket->issued, NULL);
+  pthread_cond_init(&port_ticket->revoked, NULL);
   
   return port_ticket;
 }
@@ -253,15 +259,8 @@ static void set_speed_internal (xine_stream_t *stream, int speed) {
   if( stream->audio_out ) {
     xine->port_ticket->acquire(xine->port_ticket, 1);
     
-    /*
-     * slow motion / fast forward does not play sound, drop buffered
-     * samples from the sound driver
-     */
-    if (speed != XINE_FINE_SPEED_NORMAL && speed != XINE_SPEED_PAUSE)
-      stream->audio_out->control (stream->audio_out, AO_CTRL_FLUSH_BUFFERS, NULL);
-
-    stream->audio_out->control(stream->audio_out,
-			       speed == XINE_SPEED_PAUSE ? AO_CTRL_PLAY_PAUSE : AO_CTRL_PLAY_RESUME, NULL);
+    /* inform audio_out that speed has changed - he knows what to do */
+    stream->audio_out->set_property (stream->audio_out, AO_PROP_CLOCK_SPEED, speed);
     
     xine->port_ticket->release(xine->port_ticket, 1);
   }
@@ -288,7 +287,7 @@ static void stop_internal (xine_stream_t *stream) {
   /*
    * stop demux
    */
-  if (stream->demux_plugin && stream->demux_thread) {
+  if (stream->demux_plugin && stream->demux_thread_created) {
     lprintf ("stopping demux\n");
     _x_demux_stop_thread( stream );
     lprintf ("demux stopped\n");
@@ -299,8 +298,13 @@ static void stop_internal (xine_stream_t *stream) {
 void xine_stop (xine_stream_t *stream) {
 
   pthread_mutex_lock (&stream->frontend_lock);
+  pthread_cleanup_push (mutex_cleanup, (void *) &stream->frontend_lock);
 
+  /* make sure that other threads cannot change the speed, especially pauseing the stream */
+  pthread_mutex_lock(&stream->speed_change_lock);
   stream->ignore_speed_change = 1;
+  pthread_mutex_unlock(&stream->speed_change_lock);
+
   stream->xine->port_ticket->acquire(stream->xine->port_ticket, 1);
 
   if (stream->audio_out)
@@ -321,6 +325,7 @@ void xine_stop (xine_stream_t *stream) {
   stream->xine->port_ticket->release(stream->xine->port_ticket, 1);
   stream->ignore_speed_change = 0;
   
+  pthread_cleanup_pop (0);
   pthread_mutex_unlock (&stream->frontend_lock);
 }
 
@@ -338,24 +343,32 @@ static void close_internal (xine_stream_t *stream) {
     }
   }
 
-  stream->ignore_speed_change = 1;
-  stream->xine->port_ticket->acquire(stream->xine->port_ticket, 1);
-
-  if (stream->audio_out)
-    stream->audio_out->set_property(stream->audio_out, AO_PROP_DISCARD_BUFFERS, 1);
-  if (stream->video_out)
-    stream->video_out->set_property(stream->video_out, VO_PROP_DISCARD_FRAMES, 1);
+  if( !stream->gapless_switch ) {
+    /* make sure that other threads cannot change the speed, especially pauseing the stream */
+    pthread_mutex_lock(&stream->speed_change_lock);
+    stream->ignore_speed_change = 1;
+    pthread_mutex_unlock(&stream->speed_change_lock);
 
+    stream->xine->port_ticket->acquire(stream->xine->port_ticket, 1);
+  
+    if (stream->audio_out)
+      stream->audio_out->set_property(stream->audio_out, AO_PROP_DISCARD_BUFFERS, 1);
+    if (stream->video_out)
+      stream->video_out->set_property(stream->video_out, VO_PROP_DISCARD_FRAMES, 1);
+  }
+  
   stop_internal( stream );
   
-  if (stream->video_out)
-    stream->video_out->set_property(stream->video_out, VO_PROP_DISCARD_FRAMES, 0);  
-  if (stream->audio_out)
-    stream->audio_out->set_property(stream->audio_out, AO_PROP_DISCARD_BUFFERS, 0);
-
-  stream->xine->port_ticket->release(stream->xine->port_ticket, 1);
-  stream->ignore_speed_change = 0;
+  if( !stream->gapless_switch ) {
+    if (stream->video_out)
+      stream->video_out->set_property(stream->video_out, VO_PROP_DISCARD_FRAMES, 0);  
+    if (stream->audio_out)
+      stream->audio_out->set_property(stream->audio_out, AO_PROP_DISCARD_BUFFERS, 0);
   
+    stream->xine->port_ticket->release(stream->xine->port_ticket, 1);
+    stream->ignore_speed_change = 0;
+  }
+    
   if (stream->demux_plugin) {
     _x_free_demux_plugin(stream, stream->demux_plugin);
     stream->demux_plugin = NULL;
@@ -388,6 +401,7 @@ static void close_internal (xine_stream_t *stream) {
 void xine_close (xine_stream_t *stream) {
 
   pthread_mutex_lock (&stream->frontend_lock);
+  pthread_cleanup_push (mutex_cleanup, (void *) &stream->frontend_lock);
 
   close_internal (stream);
 
@@ -401,6 +415,7 @@ void xine_close (xine_stream_t *stream) {
   if (stream->status != XINE_STATUS_QUIT)
     stream->status = XINE_STATUS_IDLE;
 
+  pthread_cleanup_pop (0);
   pthread_mutex_unlock (&stream->frontend_lock);
 }
 
@@ -492,6 +507,9 @@ xine_stream_t *xine_stream_new (xine_t *this,
   stream->spu_channel_pan_scan   = -1;
   stream->spu_channel_user       = -1;
   stream->spu_channel            = -1;
+  stream->early_finish_event     = 0;
+  stream->delay_finish_event     = 0;
+  stream->gapless_switch         = 0;
 
   stream->video_out              = vo;
   if (vo)
@@ -524,12 +542,14 @@ xine_stream_t *xine_stream_new (xine_t *this,
   pthread_mutex_init (&stream->info_mutex, NULL);
   pthread_mutex_init (&stream->meta_mutex, NULL);
   pthread_mutex_init (&stream->demux_lock, NULL);
+  pthread_mutex_init (&stream->demux_mutex, NULL);
   pthread_mutex_init (&stream->event_queues_lock, NULL);
   pthread_mutex_init (&stream->counter_lock, NULL);
   pthread_cond_init  (&stream->counter_changed, NULL);
   pthread_mutex_init (&stream->first_frame_lock, NULL);
   pthread_cond_init  (&stream->first_frame_reached, NULL);
   pthread_mutex_init (&stream->current_extra_info_lock, NULL);
+  pthread_mutex_init (&stream->speed_change_lock, NULL);
 
   /* warning: frontend_lock is a recursive mutex. it must NOT be
    * used with neither pthread_cond_wait() or pthread_cond_timedwait()
@@ -564,9 +584,24 @@ xine_stream_t *xine_stream_new (xine_t *this,
    * alloc fifos, init and start decoder threads
    */
 
-  _x_video_decoder_init (stream);
+  if (!_x_video_decoder_init (stream))
+  {
+    free(stream->audio_decoder_extra_info);
+    free(stream->current_extra_info);
+    free(stream);
+    pthread_mutex_unlock(&this->streams_lock);
+    return NULL;
+  }
 
-  _x_audio_decoder_init (stream);
+  if (!_x_audio_decoder_init (stream))
+  {
+    _x_video_decoder_shutdown(stream);
+    free(stream->audio_decoder_extra_info);
+    free(stream->current_extra_info);
+    free(stream);
+    pthread_mutex_unlock(&this->streams_lock);
+    return NULL;
+  }
 
   /*
    * osd
@@ -580,12 +615,22 @@ xine_stream_t *xine_stream_new (xine_t *this,
    * create a reference counter
    */
   stream->refcounter = _x_new_refcounter(stream, (refcounter_destructor)xine_dispose_internal);
-  
+  if (!stream->refcounter)
+  {
+    _x_video_decoder_shutdown(stream);
+    _x_audio_decoder_shutdown(stream);
+    free(stream->audio_decoder_extra_info);
+    free(stream->current_extra_info);
+    free(stream);
+    pthread_mutex_unlock(&this->streams_lock);
+    return NULL;
+  }
+
   /*
    * register stream
    */
 
-  xine_list_append_content (this->streams, stream);
+  xine_list_push_back (this->streams, stream);
 
   pthread_mutex_unlock (&this->streams_lock);
 
@@ -607,7 +652,7 @@ static void mrl_unescape(char *mrl) {
 
   for (i = 0; i < len; i++) {
     if ((mrl[i]=='%') && (i<(len-2))) {
-      int c;
+      unsigned int c;
       
       if (sscanf(&mrl[i + 1], "%02x", &c) == 1) {
 	mrl[i]= (char)c;
@@ -621,13 +666,14 @@ static void mrl_unescape(char *mrl) {
 
 void _x_flush_events_queues (xine_stream_t *stream) {
 
-  xine_event_queue_t *queue;
+  xine_list_iterator_t ite;
 
   pthread_mutex_lock (&stream->event_queues_lock);
 
   /* No events queue? */
-  for (queue = xine_list_first_content (stream->event_queues);
-       queue; queue = xine_list_next_content (stream->event_queues)) {
+  for (ite = xine_list_front (stream->event_queues);
+       ite; ite = xine_list_next (stream->event_queues, ite)) {
+    xine_event_queue_t *queue = xine_list_get_value(stream->event_queues, ite);
     pthread_mutex_lock (&queue->lock);
     pthread_mutex_unlock (&stream->event_queues_lock);
 
@@ -637,7 +683,7 @@ void _x_flush_events_queues (xine_stream_t *stream) {
      * currently executing their callback functions.
      */
     if (queue->listener_thread != NULL && !queue->callback_running) {
-      while (!xine_list_is_empty (queue->events)) {
+      while (!xine_list_empty (queue->events)) {
         pthread_cond_wait (&queue->events_processed, &queue->lock);
       }
     }
@@ -689,6 +735,8 @@ static int open_internal (xine_stream_t *stream, const char *mrl) {
      */
 
     if ((stream->input_plugin = _x_find_input_plugin (stream, input_source))) {
+      int res;
+
       xine_log (stream->xine, XINE_LOG_MSG, _("xine: found input plugin  : %s\n"),
 		stream->input_plugin->input_class->get_description(stream->input_plugin->input_class));
       if (stream->input_plugin->input_class->eject_media)
@@ -696,15 +744,23 @@ static int open_internal (xine_stream_t *stream, const char *mrl) {
       _x_meta_info_set_utf8(stream, XINE_META_INFO_INPUT_PLUGIN, 
 			    (stream->input_plugin->input_class->get_identifier (stream->input_plugin->input_class)));
 
-      if (!stream->input_plugin->open(stream->input_plugin)) {
+      res = stream->input_plugin->open(stream->input_plugin);
+      switch(res) {
+      case 1: /* Open successfull */
+	free(input_source);
+	break;
+      case -1: /* Open unsuccessfull, but correct plugin */
+	free(input_source);
+	stream->err = XINE_ERROR_INPUT_FAILED;
+	_x_flush_events_queues (stream);
+	return 0;
+      default:
 	xine_log (stream->xine, XINE_LOG_MSG, _("xine: input plugin cannot open MRL [%s]\n"),mrl);
 	stream->input_plugin->dispose(stream->input_plugin);
 	stream->input_plugin = NULL;
 	stream->err = XINE_ERROR_INPUT_FAILED;
-      } else {
-        free(input_source);
-        break;
       }
+      if ( res ) break;
     }
 
     free(input_source);
@@ -779,6 +835,8 @@ static int open_internal (xine_stream_t *stream, const char *mrl) {
 
 	  xine_log(stream->xine, XINE_LOG_MSG, _("xine: join rip input plugin\n"));
 	  input_saver = _x_rip_plugin_get_instance (stream, filename);
+	  free(filename);
+	  
 	  if( input_saver ) {
 	    stream->input_plugin = input_saver;
 	  } else {
@@ -989,15 +1047,15 @@ static int open_internal (xine_stream_t *stream, const char *mrl) {
 	  if (retval == 0) {
 	    /* the option not found */
 	    xine_log(stream->xine, XINE_LOG_MSG, _("xine: error while parsing MRL\n"));
+	    stream->err = XINE_ERROR_MALFORMED_MRL;
+	    stream->status = XINE_STATUS_IDLE;
+	    free(config_entry);
+	    return 0;
 	  } else {
             /* not permitted to change from MRL */
             xine_log(stream->xine, XINE_LOG_MSG, _("xine: changing option '%s' from MRL isn't permitted\n"),
 	      config_entry);
 	  }
-          stream->err = XINE_ERROR_MALFORMED_MRL;
-          stream->status = XINE_STATUS_IDLE;
-	  free(config_entry);
-          return 0;
 	}
 	free(config_entry);
       }
@@ -1081,11 +1139,13 @@ int xine_open (xine_stream_t *stream, const char *mrl) {
   int ret;
 
   pthread_mutex_lock (&stream->frontend_lock);
+  pthread_cleanup_push (mutex_cleanup, (void *) &stream->frontend_lock);
 
   lprintf ("open MRL:%s\n", mrl);
 
   ret = open_internal (stream, mrl);
 
+  pthread_cleanup_pop (0);
   pthread_mutex_unlock (&stream->frontend_lock);
 
   return ret;
@@ -1130,7 +1190,7 @@ static int play_internal (xine_stream_t *stream, int start_pos, int start_time)
   stream->xine->port_ticket->acquire(stream->xine->port_ticket, 1);
   
   /* only flush/discard output ports on master streams */
-  if( stream->master == stream ) {
+  if( stream->master == stream && !stream->gapless_switch) {
     /* discard audio/video buffers to get engine going and take the lock faster */
     if (stream->audio_out)
       stream->audio_out->set_property(stream->audio_out, AO_PROP_DISCARD_BUFFERS, 1);
@@ -1159,7 +1219,7 @@ static int play_internal (xine_stream_t *stream, int start_pos, int start_time)
 					     stream->demux_thread_running);
 
   /* only flush/discard output ports on master streams */
-  if( stream->master == stream ) {
+  if( stream->master == stream && !stream->gapless_switch) {
     if (stream->audio_out)
       stream->audio_out->set_property(stream->audio_out, AO_PROP_DISCARD_BUFFERS, 0);
     if (stream->video_out)
@@ -1213,11 +1273,17 @@ int xine_play (xine_stream_t *stream, int start_pos, int start_time) {
   int ret;
 
   pthread_mutex_lock (&stream->frontend_lock);
+  pthread_cleanup_push (mutex_cleanup, (void *) &stream->frontend_lock);
 
+  stream->delay_finish_event = 0;
+  
   ret = play_internal (stream, start_pos, start_time);
   if( stream->slave && (stream->slave_affection & XINE_MASTER_SLAVE_PLAY) )
     xine_play (stream->slave, start_pos, start_time);
+  
+  stream->gapless_switch = 0;
 
+  pthread_cleanup_pop (0);
   pthread_mutex_unlock (&stream->frontend_lock);
   
   return ret;
@@ -1231,6 +1297,7 @@ int xine_eject (xine_stream_t *stream) {
     return 0;
   
   pthread_mutex_lock (&stream->frontend_lock);
+  pthread_cleanup_push (mutex_cleanup, (void *) &stream->frontend_lock);
 
   status = 0;
   /* only eject, if we are stopped OR a different input plugin is playing */
@@ -1241,13 +1308,15 @@ int xine_eject (xine_stream_t *stream) {
     status = stream->eject_class->eject_media (stream->eject_class);
   }
 
+  pthread_cleanup_pop (0);
   pthread_mutex_unlock (&stream->frontend_lock);
+  
   return status;
 }
 
 void xine_dispose_internal (xine_stream_t *stream) {
 
-  xine_stream_t *s;
+  xine_list_iterator_t *ite;
 
   lprintf("stream: %p\n", stream);
   pthread_mutex_destroy (&stream->info_mutex);
@@ -1257,6 +1326,7 @@ void xine_dispose_internal (xine_stream_t *stream) {
   pthread_mutex_destroy (&stream->event_queues_lock);
   pthread_mutex_destroy (&stream->current_extra_info_lock);
   pthread_cond_destroy  (&stream->counter_changed);
+  pthread_mutex_destroy (&stream->demux_mutex);
   pthread_mutex_destroy (&stream->demux_lock);
   pthread_mutex_destroy (&stream->first_frame_lock);
   pthread_cond_destroy  (&stream->first_frame_reached);
@@ -1264,12 +1334,9 @@ void xine_dispose_internal (xine_stream_t *stream) {
   stream->metronom->exit (stream->metronom);
 
   pthread_mutex_lock(&stream->xine->streams_lock);
-  for (s = xine_list_first_content(stream->xine->streams);
-       s; s = xine_list_next_content(stream->xine->streams)) {
-    if (s == stream) {
-      xine_list_delete_current (stream->xine->streams);
-      break;
-    }
+  ite = xine_list_find(stream->xine->streams, stream);
+  if (ite) {
+    xine_list_remove(stream->xine->streams, ite);
   }
   pthread_mutex_unlock(&stream->xine->streams_lock);
 
@@ -1320,12 +1387,13 @@ void xine_exit (xine_t *this) {
   xprintf (this, XINE_VERBOSITY_DEBUG, "xine_exit: bye!\n");
 
   for (i = 0; i < XINE_LOG_NUM; i++)
-    this->log_buffers[i]->dispose (this->log_buffers[i]);
+    if ( this->log_buffers[i] )
+      this->log_buffers[i]->dispose (this->log_buffers[i]);
 
   _x_dispose_plugins (this);
 
   if(this->streams) {
-    xine_list_free(this->streams);
+    xine_list_delete(this->streams);
     pthread_mutex_destroy(&this->streams_lock);
   }
 
@@ -1341,18 +1409,17 @@ void xine_exit (xine_t *this) {
 #if defined(WIN32)
   WSACleanup();
 #endif
-  
+
   free (this);
 }
 
 xine_t *xine_new (void) {
   xine_t      *this;
-  int          i;
 
 #ifdef WIN32
     WSADATA Data;
     int i_err;
-#endif /*  WIN32 */
+#endif
 
   this = xine_xmalloc (sizeof (xine_t));
   if (!this)
@@ -1381,9 +1448,7 @@ xine_t *xine_new (void) {
   /*
    * log buffers
    */
-
-  for (i = 0; i < XINE_LOG_NUM; i++)
-    this->log_buffers[i] = _x_new_scratch_buffer (150);
+  memset(this->log_buffers, 0, sizeof(this->log_buffers));
 
 
 #ifdef WIN32
@@ -1452,13 +1517,16 @@ static void config_save_cb (void *this_gen, xine_cfg_entry_t *entry) {
        strcmp(entry->str_value, xine_get_homedir()) == 0 ||
        strcmp(entry->str_value, homedir_trail_slash) == 0)) {
     xine_stream_t *stream;
+    xine_list_iterator_t ite;
     
     xine_log(this, XINE_LOG_MSG,
 	     _("xine: The specified save_dir \"%s\" might be a security risk.\n"), entry->str_value);
     
     pthread_mutex_lock(&this->streams_lock);
-    if ((stream = (xine_stream_t *)xine_list_first_content(this->streams)))
+    if ( (ite = xine_list_front(this->streams)) ) {
+      stream = xine_list_get_value(this->streams, ite);
       _x_message(stream, XINE_MSG_SECURITY, _("The specified save_dir might be a security risk."), NULL);
+    }
     pthread_mutex_unlock(&this->streams_lock);
   }
   
@@ -1508,9 +1576,9 @@ void xine_init (xine_t *this) {
   /*
    * save directory
    */
-  this->save_path  = this->config->register_string (
+  this->save_path  = this->config->register_filename (
       this->config, 
-      "media.capture.save_dir", "",
+      "media.capture.save_dir", "", XINE_CONFIG_STRING_IS_DIRECTORY_NAME,
       _("directory for saving streams"),
       _("When using the stream save feature, files will be written only into this directory.\n"
 	"This setting is security critical, because when changed to a different directory, xine "
@@ -1533,15 +1601,28 @@ void xine_init (xine_t *this) {
 	"messed up xine."),
       XINE_CONFIG_SECURITY, NULL, this);
 
+  /*
+   * timeout for network I/O to avoid freezes
+   */
+  this->config->register_num(this->config,
+      "media.network.timeout", 30,
+      _("Timeout for network stream reading (in seconds)"),
+      _("Specifies the timeout when reading from network streams, in seconds. "
+	"Too low values might stop streaming when the source is slow or the "
+	"bandwidth is occupied, too high values will freeze the player if the "
+	"connection is lost."),
+      0, NULL, this);
+
   /*
    * keep track of all opened streams
    */
   this->streams = xine_list_new();
 
   /*
-   * streams lock
+   * locks
    */
   pthread_mutex_init (&this->streams_lock, NULL);
+  pthread_mutex_init (&this->log_lock, NULL);
   
   /*
    * start metronom clock
@@ -1637,18 +1718,20 @@ int xine_get_status (xine_stream_t *stream) {
  */
 
 void _x_set_fine_speed (xine_stream_t *stream, int speed) {
+  pthread_mutex_lock(&stream->speed_change_lock);
 
-  if (stream->ignore_speed_change)
-    return;
-
-  if (speed <= XINE_SPEED_PAUSE)
-    speed = XINE_SPEED_PAUSE;
-
-  xprintf (stream->xine, XINE_VERBOSITY_DEBUG, "set_speed %d\n", speed);
-  set_speed_internal (stream, speed);
+  if (!stream->ignore_speed_change)
+  {
+    if (speed <= XINE_SPEED_PAUSE)
+      speed = XINE_SPEED_PAUSE;
   
-  if (stream->slave && (stream->slave_affection & XINE_MASTER_SLAVE_SPEED))
-    set_speed_internal (stream->slave, speed);
+    xprintf (stream->xine, XINE_VERBOSITY_DEBUG, "set_speed %d\n", speed);
+    set_speed_internal (stream, speed);
+    
+    if (stream->slave && (stream->slave_affection & XINE_MASTER_SLAVE_SPEED))
+      set_speed_internal (stream->slave, speed);
+  }
+  pthread_mutex_unlock(&stream->speed_change_lock);
 }
 
 int _x_get_fine_speed (xine_stream_t *stream) {
@@ -1884,10 +1967,22 @@ const char *const *xine_get_log_names (xine_t *this) {
   return log_sections;
 }
 
+static inline void check_log_alloc (xine_t *this, int buf)
+{
+  pthread_mutex_lock (&this->log_lock);
+
+  if ( ! this->log_buffers[buf] )
+    this->log_buffers[buf] = _x_new_scratch_buffer(150);
+
+  pthread_mutex_unlock (&this->log_lock);
+}
+
 void xine_log (xine_t *this, int buf, const char *format, ...) {
   va_list argp;
   char    buffer[SCRATCH_LINE_LEN_MAX];
   
+  check_log_alloc (this, buf);
+
   va_start (argp, format);
   this->log_buffers[buf]->scratch_printf (this->log_buffers[buf], format, argp);
   va_end(argp);
@@ -1900,12 +1995,23 @@ void xine_log (xine_t *this, int buf, const char *format, ...) {
   }  
 }
 
+void xine_vlog(xine_t *this, int buf, const char *format, 
+                va_list args)
+{
+  check_log_alloc (this, buf);
+
+  this->log_buffers[buf]->scratch_printf(this->log_buffers[buf], format, args);
+}
+
 const char *const *xine_get_log (xine_t *this, int buf) {
 
   if(buf >= XINE_LOG_NUM)
     return NULL;
 
-  return this->log_buffers[buf]->get_content (this->log_buffers[buf]);
+  if ( this->log_buffers[buf] )
+    return this->log_buffers[buf]->get_content (this->log_buffers[buf]);
+  else
+    return NULL;
 }
 
 void xine_register_log_cb (xine_t *this, xine_log_cb_t cb, void *user_data) {
